Module: Reactor::Permission
- Defined in:
- lib/reactor/permission.rb
Overview
This module adds #permission method to obj that act as a gateway for permission checking, see documentation for [Permission::PermissionProxy] for more details. Including this module also alters typical ActiveRecord call chain, so that it becomes:
- permission checking
- validations (if [Reactor::Validations] is included)
- callbacks (before_, around_, after_*)
Therefore if the user lacks permissions no futher actions are executed.
Defined Under Namespace
Modules: Base Classes: PermissionLookup, PermissionProxy